.sidebar-section {
    margin-top: 2rem;
}

.sidebar-heading {
    font-size: 0.75rem;
    font-weight: 600;
    text-transform: uppercase;
    color: var(--color-foreground-muted);
    margin-bottom: 0.5rem;
}

.sidebar-section ul {
    list-style: none;
    padding-left: 0;
}

.sidebar-section li {
    margin: 0.4rem 0;
}
